
/*
 *	vtrans.hxx
 *	vector <-> sequence transformation using 'C' semantics
 *	(REMIND: see also vector.hxx)
 */

#ifndef _VTRANS_HXX
#define _VTRANS_HXX 1

/*
 *	piDim	array dimensions, e.g. C_array[10][2] <=> {2,10 }
 *		reversed: C_array[10][2] <=> { 10, 2 }
 *	pi	vector, pi[x]<piDim[x], 0 <= x < cDim
 *	cDim	count dimensions
 *	
 *	ltrans( {10,2} , {5,1}, 2) => 15
 *	ltransr( {2,10} , {1,5}, 2) => 15
 *	vtrans(..., {10,2} , 15, 2) => { 5,1 }
 *	
 *	NOTE: DO NOT ASSUME DETAILS OF TRANSFORMATION FUNCTION
 */

/* vec => lin	*/
int ltrans(const int *piDim, const int *pi, int cDim);	
int ltransr(const int *piDim, const int *pi, int cDim);	
/* lin => vec	*/
void vtrans(int *pi, const int *piDim, int iLin, int cDim);

int lcnt(const int *piDim, int cDim);	/* cnt for ltrans array	*/
/*
 *	'increment' vector equivalent of linear representation
 *
 *	returns index of topmost increment:
 *	0 = pi[0] incremented
 *	1 = pi[1] incremented
 *	...
 *	cDim = overflow (or carry)
 */
int inc(int *pi, const int *piDim, int cDim);
void copy(int *piDst, const int *piSrc, int cDim);

/*
 *	vL == vR
 */
int eq(int *piL, int *piR, int cDim);

/*
 *	min(), max() - min/max of two vectors 
 */
const int *min(const int *piL, const int *piR, const int *piDim, int cDim);
const int *max(const int *piL, const int *piR, const int *piDim, int cDim);

/*
 *	floor() - floor vector 
 *
 *	c=2: 10(11) -> 10(00)
 */
void floor(int *pi, int c, const int *piDim, int cDim);

/*
 *	ceill() - ceill vector 
 *
 *	d={1233}, c=2: 10(10) -> 10(22)
 */
void ceill(int *pi, int c, const int *piDim, int cDim);

#endif
