
#ifndef _GPIB_GUMI_HXX
#define _GPIB_GUMI_HXX

#include "umic/umi.hxx"

/**
 * \file
 * GPIB error codes, status bits, and line bits
 */

/** namespace for ines GPIB */
namespace INdepGpib100 {

  /** controller interface error codes */
  enum ErrE {
    E_OK = 0,
    E_HWFAIL,		/**< unexpected state at hardware layer */
    E_ADR,		/**< addressing scheme wrong		*/
    E_SAC,		/**< system control required		*/
    E_NCIC,		/**< interface not controller in charge	*/
    E_CIC,		/**< device function called while CIC	*/
    E_TMO,		/**< timeout				*/
    E_STATE,		/**< wrong state			*/
    E_MEM,		/**< allocation failure			*/
    E_TRANSFAIL,	/**< previous message of transaction failed 	*/
    E_INUSE,		/**< interface already in use	 	*/
    E_MSG,		/**< incompatible message format        */
    E_NOL,		/**< no listeners detected		*/
    E_DCL,		/**< DCAS during data transfer, clr E_DCAS */
    E_CAP,		/**< no capability for operation	*/

    CNT_E
  };

  /** status enum */
  enum StatE {
    /* direct bits */
    F_LA =  0x01,	/**< LAxS					*/
    F_TA =  0x02,	/**< TAxS					*/
    F_CIC = 0x04,	/**< CACS|CSBS				*/
    F_ATN = 0x08,	/**< qualifies the flags above		*/
    F_SC =  0x10,	/**< SACS					*/
    F_REM = 0x20,	/**< REMS|RWLS				*/
    F_LOK = 0x40,	/**< LWLS|RWLS				*/

    /* event bits */
    E_SRQI = 0x100,	/**< SRQ seen				*/
    E_DTAS = 0x200,	/**< DTAS seen				*/
    E_DCAS = 0x400	/**< DCAS seen				*/
  };

}


#endif 

