#ifndef _CARD_H
#define _CARD_H

#include <linux/spinlock.h>
#include <linux/wait.h>
#include <asm/atomic.h>
#include "fixme.h"
#include "umic/umi.h"
#include "umic/umi_c.h"


  /*
   * Channel object
   */
  struct CardStruct;
  struct ChanStruct {
    unsigned long _flag;			/* flags			*/
    struct CardStruct *_pca;			/* host				*/
    const struct UmiDriverDesc *_pUdd;		/* description of the driver	*/
    int _iChanOfDriver;				/* channel # of driver		*/
    int _iChanOfCard;				/* channel # of card		*/
    int _iDriverOfCard;				/* driver # of card		*/
    const char *_szStat;			/* current status		*/
  };
  typedef struct ChanStruct Chan;

  #define CHN_FLG_PUT 0	/* put path executing		*/
  #define CHN_FLG_GET 1	/* get path executing		*/

  #define pcn_init(p,c,d,i,j,k) ((p)->_flag=0, (p)->_pca=(c), (p)->_pUdd=(d), (p)->_iChanOfDriver=(i), \
      (p)->_iChanOfCard=(j), (p)->_iDriverOfCard=(k), (p)->_szStat="")
  /*
  #define pcn_init(p,c,d) ((p)->_flag=0, (p)->_pca=(c), \
      (p)->_szStat="")
  */

  #define pcn_isNonBlocking(p) (0)
  #define pcn_lockGet(p) (test_and_set_bit(CHN_FLG_GET, &(p)->_flag)==0)
  #define pcn_lockPut(p) (test_and_set_bit(CHN_FLG_PUT, &(p)->_flag)==0)
  #define pcn_unlockGet(p, i) (clear_bit(CHN_FLG_GET, &(p)->_flag), (i))
  #define pcn_unlockPut(p, i) (clear_bit(CHN_FLG_PUT, &(p)->_flag), (i))
  #define pcn_stat(pcn, sz) (pcn)->_szStat=(sz)
  #define pcn_iChan(p) (0)


  /*
   * per adaptor card control struct
   * base class to be overloaded by adaptor 
   * type (bus, usb, ...)
   */
  struct LumiModStruct;
  struct CardStruct {
    struct LumiModStruct *_pdm;		/* module this card belongs to	*/

    /* channels */
    Chan _mpcn[MAX_CACN];

    /* 
     * to be set by the derived class
     * access always serialized, args passed are user space pointers
     */
    int (*_pfPutDown)(struct CardStruct *, struct ChanStruct *, unsigned long arg);	
    int (*_pfGetUp)(struct CardStruct *, struct ChanStruct *, unsigned long arg);	

    int (*_pfOpen)(struct CardStruct *, struct ChanStruct *);	
    int (*_pfClose)(struct CardStruct *, struct ChanStruct *);	

    char* (*_pfPrint)( struct CardStruct *pca, char* pchBuf, int c);

  } ;
  typedef struct CardStruct Card;
  extern void pca_init(Card *pca, struct LumiModStruct *pdm);
  #define pca_pdm(p) ((p)->_pdm)
  #define pca_pcn(p, i) ((p)->_mpcn+(i))

  #define pcn_putdown(p, arg) ((((p)->_pca->_pfPutDown))((p)->_pca, (p), arg))
  #define pcn_getup(p, arg) ((((p)->_pca->_pfGetUp))((p)->_pca, (p), arg))
  #define pcn_open(p) ((((p)->_pca->_pfOpen))((p)->_pca, (p)))
  #define pcn_close(p) ((((p)->_pca->_pfClose))((p)->_pca, (p)))

  /*
   * user/kernel buffer copy
   */
  extern int bufPutDown( struct UmiMemStruct *pudU, char *pch, size_t sMax, size_t *psCur);
  extern int bufGetUp( struct UmiMemStruct *pudU, char *pchC, size_t s);
  

#endif
